@@ -293,11 +293,7 @@ class MetricsLayerQuery(MetricsLayerBase):
     # Code for formatting values
     def get_sql(self, field, alias: str = None, use_symmetric: bool = False):
         if use_symmetric:
-            query = field.sql_query(
-                query_type=self.query_type,
-                query_base_view=self.design.base_view_name,
-                joins=self.design.joins(),
-            )
+            query = field.sql_query(query_type=self.query_type, functional_pk=self.design.functional_pk())
         else:
             query = field.sql_query(query_type=self.query_type)
         return self.sql(query, alias)
